@@ -51,7 +51,7 @@ class Array
   #    b.uniq { |s| s.first } # => [["student", "sam"], ["teacher", "matz"]]
   #
   def uniq(&block)
-    ary = self.dup
+    ary = self[0..-1]
     ary.uniq!(&block)
     ary
   end
@@ -88,6 +88,22 @@ class Array
     array
   end
 
+  ##
+  # call-seq:
+  #    ary.difference(other_ary1, other_ary2, ...)   -> new_ary
+  #
+  # Returns a new array that is a copy of the original array, removing all
+  # occurrences of any item that also appear in +other_ary+. The order is
+  # preserved from the original array.
+  #
+  def difference(*args)
+    ary = self
+    args.each do |x|
+      ary = ary - x
+    end
+    ary
+  end
+
   ##
   # call-seq:
   #    ary | other_ary     -> new_ary
@@ -134,7 +150,7 @@ class Array
   #    [ 1, 1, 3, 5 ] & [ 1, 2, 3 ]   #=> [ 1, 3 ]
   #
   def &(elem)
-    raise TypeError, "can't convert #{elem.class} into Array" unless elem.class == Array
+    raise TypeError, "cannot convert #{elem.class} into Array" unless elem.class == Array
 
     hash = {}
     array = []
@@ -157,6 +173,65 @@ class Array
     array
   end
 
+  ##
+  # call-seq:
+  #    ary.intersection(other_ary,...)  -> new_ary
+  #
+  # Set Intersection---Returns a new array containing elements common to
+  # this array and <i>other_ary</i>s, removing duplicates. The order is
+  # preserved from the original array.
+  #
+  #    [1, 2, 3].intersection([3, 4, 1], [1, 3, 5])  #=> [1, 3]
+  #
+  def intersection(*args)
+    ary = self
+    args.each do |x|
+      ary = ary & x
+    end
+    ary
+  end
+
+  ##
+  # call-seq:
+  #   ary.intersect?(other_ary)   -> true or false
+  #
+  # Returns +true+ if the array and +other_ary+ have at least one element in
+  # common, otherwise returns +false+.
+  #
+  #    a = [ 1, 2, 3 ]
+  #    b = [ 3, 4, 5 ]
+  #    c = [ 5, 6, 7 ]
+  #    a.intersect?(b)   #=> true
+  #    a.intersect?(c)   #=> false
+  def intersect?(ary)
+    raise TypeError, "cannot convert #{ary.class} into Array" unless ary.class == Array
+
+    hash = {}
+    if self.length > ary.length
+      shorter = ary
+      longer = self
+    else
+      shorter = self
+      longer = ary
+    end
+    idx = 0
+    len = shorter.size
+    while idx < len
+      hash[shorter[idx]] = true
+      idx += 1
+    end
+    idx = 0
+    len = size
+    while idx < len
+      v = longer[idx]
+      if hash[v]
+        return true
+      end
+      idx += 1
+    end
+    false
+  end
+
   ##
   # call-seq:
   #    ary.flatten -> new_ary
@@ -164,7 +239,7 @@ class Array
   #
   # Returns a new array that is a one-dimensional flattening of this
   # array (recursively). That is, for every element that is an array,
-  # extract its elements into the new array.  If the optional
+  # extract its elements into the new array. If the optional
   # <i>level</i> argument determines the level of recursion to flatten.
   #
   #    s = [ 1, 2, 3 ]           #=> [1, 2, 3]
@@ -175,7 +250,7 @@ class Array
   #    a.flatten(1)              #=> [1, 2, 3, [4, 5]]
   #
   def flatten(depth=nil)
-    res = dup
+    res = Array.new(self)
     res.flatten! depth
     res
   end
@@ -187,7 +262,7 @@ class Array
   #
   # Flattens +self+ in place.
   # Returns <code>nil</code> if no modifications were made (i.e.,
-  # <i>ary</i> contains no subarrays.)  If the optional <i>level</i>
+  # <i>ary</i> contains no subarrays.) If the optional <i>level</i>
   # argument determines the level of recursion to flatten.
   #
   #    a = [ 1, 2, [3, [4, 5] ] ]
@@ -219,41 +294,6 @@ class Array
     end
   end
 
-  ##
-  # call-seq:
-  #    ary.compact     -> new_ary
-  #
-  # Returns a copy of +self+ with all +nil+ elements removed.
-  #
-  #    [ "a", nil, "b", nil, "c", nil ].compact
-  #                      #=> [ "a", "b", "c" ]
-  #
-  def compact
-    result = self.dup
-    result.compact!
-    result
-  end
-
-  ##
-  # call-seq:
-  #    ary.compact!    -> ary  or  nil
-  #
-  # Removes +nil+ elements from the array.
-  # Returns +nil+ if no changes were made, otherwise returns
-  # <i>ary</i>.
-  #
-  #    [ "a", nil, "b", nil, "c" ].compact! #=> [ "a", "b", "c" ]
-  #    [ "a", "b", "c" ].compact!           #=> nil
-  #
-  def compact!
-    result = self.select { |e| !e.nil? }
-    if result.size == self.size
-      nil
-    else
-      self.replace(result)
-    end
-  end
-
   # for efficiency
   def reverse_each(&block)
     return to_enum :reverse_each unless block
@@ -273,7 +313,7 @@ class Array
   #     ary.fetch(index) { |index| block }  -> obj
   #
   #  Tries to return the element at position +index+, but throws an IndexError
-  #  exception if the referenced +index+ lies outside of the array bounds.  This
+  #  exception if the referenced +index+ lies outside of the array bounds. This
   #  error can be prevented by supplying a second argument, which will act as a
   #  +default+ value.
   #
@@ -291,7 +331,7 @@ class Array
   #
 
   def fetch(n, ifnone=NONE, &block)
-    warn "block supersedes default value argument" if !n.nil? && ifnone != NONE && block
+    #warn "block supersedes default value argument" if !n.nil? && ifnone != NONE && block
 
     idx = n
     if idx < 0
@@ -342,11 +382,10 @@ class Array
 
   def fill(arg0=nil, arg1=nil, arg2=nil, &block)
     if arg0.nil? && arg1.nil? && arg2.nil? && !block
-      raise ArgumentError, "wrong number of arguments (0 for 1..3)"
+      raise ArgumentError, "wrong number of arguments (given 0, expected 1..3)"
     end
 
     beg = len = 0
-    ary = []
     if block
       if arg0.nil? && arg1.nil? && arg2.nil?
         # ary.fill { |index| block }                    -> ary
@@ -408,57 +447,6 @@ class Array
     self
   end
 
-  ##
-  #  call-seq:
-  #     ary.rotate(count=1)    -> new_ary
-  #
-  #  Returns a new array by rotating +self+ so that the element at +count+ is
-  #  the first element of the new array.
-  #
-  #  If +count+ is negative then it rotates in the opposite direction, starting
-  #  from the end of +self+ where +-1+ is the last element.
-  #
-  #     a = [ "a", "b", "c", "d" ]
-  #     a.rotate         #=> ["b", "c", "d", "a"]
-  #     a                #=> ["a", "b", "c", "d"]
-  #     a.rotate(2)      #=> ["c", "d", "a", "b"]
-  #     a.rotate(-3)     #=> ["b", "c", "d", "a"]
-
-  def rotate(count=1)
-    ary = []
-    len = self.length
-
-    if len > 0
-      idx = (count < 0) ? (len - (~count % len) - 1) : (count % len) # rotate count
-      len.times do
-        ary << self[idx]
-        idx += 1
-        idx = 0 if idx > len-1
-      end
-    end
-    ary
-  end
-
-  ##
-  #  call-seq:
-  #     ary.rotate!(count=1)   -> ary
-  #
-  #  Rotates +self+ in place so that the element at +count+ comes first, and
-  #  returns +self+.
-  #
-  #  If +count+ is negative then it rotates in the opposite direction, starting
-  #  from the end of the array where +-1+ is the last element.
-  #
-  #     a = [ "a", "b", "c", "d" ]
-  #     a.rotate!        #=> ["b", "c", "d", "a"]
-  #     a                #=> ["b", "c", "d", "a"]
-  #     a.rotate!(2)     #=> ["d", "a", "b", "c"]
-  #     a.rotate!(-3)    #=> ["a", "b", "c", "d"]
-
-  def rotate!(count=1)
-    self.replace(self.rotate(count))
-  end
-
   ##
   #  call-seq:
   #     ary.delete_if { |item| block }  -> ary
@@ -551,7 +539,7 @@ class Array
   #  the given condition in O(log n) where n is the size of the array.
   #
   #  You can use this method in two use cases: a find-minimum mode and
-  #  a find-any mode.  In either case, the elements of the array must be
+  #  a find-any mode. In either case, the elements of the array must be
   #  monotone (or sorted) with respect to the block.
   #
   #  In find-minimum mode (this is a good choice for typical use case),
@@ -563,7 +551,7 @@ class Array
   #  - the block returns true for any element whose index is greater
   #    than or equal to i.
   #
-  #  This method returns the i-th element.  If i is equal to ary.size,
+  #  This method returns the i-th element. If i is equal to ary.size,
   #  it returns nil.
   #
   #     ary = [0, 4, 7, 10, 12]
@@ -582,7 +570,7 @@ class Array
   #    j <= k < ary.size.
   #
   #  Under this condition, this method returns any element whose index
-  #  is within i...j.  If i is equal to j (i.e., there is no element
+  #  is within i...j. If i is equal to j (i.e., there is no element
   #  that satisfies the block), this method returns nil.
   #
   #     ary = [0, 4, 7, 10, 12]
@@ -592,7 +580,7 @@ class Array
   #     ary.bsearch {|x| 4 - (x / 2).truncate } #=> nil
   #
   #  You must not mix the two modes at a time; the block must always
-  #  return either true/false, or always return a number.  It is
+  #  return either true/false, or always return a number. It is
   #  undefined which value is actually picked up at each iteration.
 
   def bsearch(&block)
@@ -652,37 +640,6 @@ class Array
     satisfied ? low : nil
   end
 
-  ##
-  #  call-seq:
-  #     ary.delete_if { |item| block }  -> ary
-  #     ary.delete_if                   -> Enumerator
-  #
-  #  Deletes every element of +self+ for which block evaluates to +true+.
-  #
-  #  The array is changed instantly every time the block is called, not after
-  #  the iteration is over.
-  #
-  #  See also Array#reject!
-  #
-  #  If no block is given, an Enumerator is returned instead.
-  #
-  #     scores = [ 97, 42, 75 ]
-  #     scores.delete_if {|score| score < 80 }   #=> [97]
-
-  def delete_if(&block)
-    return to_enum :delete_if unless block
-
-    idx = 0
-    while idx < self.size do
-      if block.call(self[idx])
-        self.delete_at(idx)
-      else
-        idx += 1
-      end
-    end
-    self
-  end
-
   ##
   #  call-seq:
   #     ary.keep_if { |item| block } -> ary
@@ -702,7 +659,6 @@ class Array
     return to_enum :keep_if unless block
 
     idx = 0
-    len = self.size
     while idx < self.size do
       if block.call(self[idx])
         idx += 1
@@ -751,7 +707,7 @@ class Array
   #  <code>==</code> to +obj+.
   #
   #  If a block is given instead of an argument, returns the _index_ of the
-  #  first object for which the block returns +true+.  Returns +nil+ if no
+  #  first object for which the block returns +true+. Returns +nil+ if no
   #  match is found.
   #
   # ISO 15.2.12.5.14
@@ -815,12 +771,11 @@ class Array
   #  a.permutation(0).to_a #=> [[]] # one permutation of length 0
   #  a.permutation(4).to_a #=> []   # no permutations of length 4
   def permutation(n=self.size, &block)
-    size = self.size
     return to_enum(:permutation, n) unless block
-    return if n > size
+    size = self.size
     if n == 0
-       yield []
-    else
+      yield []
+    elsif 0 < n && n <= size
       i = 0
       while i<size
         result = [self[i]]
@@ -835,6 +790,7 @@ class Array
         i += 1
       end
     end
+    self
   end
 
   ##
@@ -861,18 +817,17 @@ class Array
   #    a.combination(5).to_a  #=> []   # no combinations of length 5
 
   def combination(n, &block)
-    size = self.size
     return to_enum(:combination, n) unless block
-    return if n > size
+    size = self.size
     if n == 0
-       yield []
+      yield []
     elsif n == 1
       i = 0
       while i<size
         yield [self[i]]
         i += 1
       end
-    else
+    elsif n <= size
       i = 0
       while i<size
         result = [self[i]]
@@ -882,6 +837,7 @@ class Array
         i += 1
       end
     end
+    self
   end
 
   ##
@@ -903,8 +859,8 @@ class Array
     column_count = nil
     self.each do |row|
       raise TypeError unless row.is_a?(Array)
-      column_count ||= row.count
-      raise IndexError, 'element size differs' unless column_count == row.count
+      column_count ||= row.size
+      raise IndexError, 'element size differs' unless column_count == row.size
     end
 
     Array.new(column_count) do |column_index|
@@ -917,7 +873,7 @@ class Array
   #    ary.to_h                ->   Hash
   #    ary.to_h{|item| ... }   ->   Hash
   #
-  # Returns the result of interpreting <i>aray</i> as an array of
+  # Returns the result of interpreting <i>array</i> as an array of
   # <tt>[key, value]</tt> pairs. If a block is given, it should
   # return <tt>[key, value]</tt> pairs to construct a hash.
   #
@@ -936,4 +892,128 @@ class Array
     end
     h
   end
+
+  alias append push
+  alias prepend unshift
+  alias filter! select!
+
+  ##
+  # call-seq:
+  #   ary.product(*arys)                  ->   array
+  #   ary.product(*arys) { |item| ... }   ->   self
+  def product(*arys, &block)
+    size = arys.size
+    i = size
+    while i > 0
+      i -= 1
+      unless arys[i].kind_of?(Array)
+        raise TypeError, "no implicit conversion into Array"
+      end
+    end
+
+    i = size
+    total = self.size
+    total *= arys[i -= 1].size while i > 0
+
+    if block
+      result = self
+      list = ->(*, e) { block.call e }
+      class << list; alias []= call; end
+    else
+      result = [nil] * total
+      list = result
+    end
+
+    i = 0
+    while i < total
+      group = [nil] * (size + 1)
+      j = size
+      n = i
+      while j > 0
+        j -= 1
+        a = arys[j]
+        b = a.size
+        group[j + 1] = a[n % b]
+        n /= b
+      end
+      group[0] = self[n]
+      list[i] = group
+      i += 1
+    end
+
+    result
+  end
+
+  ##
+  # call-seq:
+  #   ary.repeated_combination(n) { |combination| ... }   ->   self
+  #   ary.repeated_combination(n)                         ->   enumerator
+  #
+  # A +combination+ method that contains the same elements.
+  def repeated_combination(n, &block)
+    raise TypeError, "no implicit conversion into Integer" unless 0 <=> n
+    return to_enum(:repeated_combination, n) unless block
+    __repeated_combination(n, false, &block)
+  end
+
+  ##
+  # call-seq:
+  #   ary.repeated_permutation(n) { |permutation| ... }   ->   self
+  #   ary.repeated_permutation(n)                         ->   enumerator
+  #
+  # A +permutation+ method that contains the same elements.
+  def repeated_permutation(n, &block)
+    raise TypeError, "no implicit conversion into Integer" unless 0 <=> n
+    return to_enum(:repeated_permutation, n) unless block
+    __repeated_combination(n, true, &block)
+  end
+
+  def __repeated_combination(n, permutation, &block)
+    case n
+    when 0
+      yield []
+    when 1
+      i = 0
+      while i < self.size
+        yield [self[i]]
+        i += 1
+      end
+    else
+      if n > 0
+        v = [0] * n
+        while true
+          tmp = [nil] * n
+          i = 0
+          while i < n
+            tmp[i] = self[v[i]]
+            i += 1
+          end
+
+          yield tmp
+
+          tmp = self.size
+          i = n - 1
+          while i >= 0
+            v[i] += 1
+            break if v[i] < tmp
+            i -= 1
+          end
+          break unless v[0] < tmp
+          i = 1
+          while i < n
+            unless v[i] < tmp
+              if permutation
+                v[i] = 0
+              else
+                v[i] = v[i - 1]
+              end
+            end
+            i += 1
+          end
+        end
+      end
+    end
+
+    self
+  end
 end
