@@ -132,16 +132,20 @@ class Enumerator
     @stop_exc = false
   end
 
-  attr_accessor :obj, :meth, :args, :kwd
-  attr_reader :fib
-
   def initialize_copy(obj)
     raise TypeError, "can't copy type #{obj.class}" unless obj.kind_of? Enumerator
-    raise TypeError, "can't copy execution context" if obj.fib
-    @obj = obj.obj
-    @meth = obj.meth
-    @args = obj.args
-    @kwd = obj.kwd
+    raise TypeError, "can't copy execution context" if obj.instance_eval{@fib}
+    meth = args = kwd = fib = nil
+    obj.instance_eval {
+      obj = @obj
+      meth = @meth
+      args = @args
+      kwd = @kwd
+    }
+    @obj = obj
+    @meth = meth
+    @args = args
+    @kwd = kwd
     @fib = nil
     @lookahead = nil
     @feedvalue = nil
@@ -169,7 +173,7 @@ class Enumerator
     end
 
     n = offset - 1
-    enumerator_block_call do |*i|
+    __enumerator_block_call do |*i|
       n += 1
       block.call i.__svalue, n
     end
@@ -219,7 +223,7 @@ class Enumerator
   def with_object(object, &block)
     return to_enum(:with_object, object) unless block
 
-    enumerator_block_call do |i|
+    __enumerator_block_call do |i|
       block.call [i,object]
     end
     object
@@ -234,6 +238,14 @@ class Enumerator
     end
   end
 
+  def size
+    if @size
+      @size
+    elsif @obj.respond_to?(:size)
+      @obj.size
+    end
+  end
+
   ##
   # call-seq:
   #   enum.each { |elm| block }                    -> obj
@@ -274,23 +286,23 @@ class Enumerator
     obj = self
     if 0 < argv.length
       obj = self.dup
-      args = obj.args
+      args = obj.instance_eval{@args}
       if !args.empty?
         args = args.dup
         args.concat argv
       else
         args = argv.dup
       end
-      obj.args = args
+      obj.instance_eval{@args = args}
     end
     return obj unless block
-    enumerator_block_call(&block)
+    __enumerator_block_call(&block)
   end
 
-  def enumerator_block_call(&block)
+  def __enumerator_block_call(&block)
     @obj.__send__ @meth, *@args, **@kwd, &block
   end
-  private :enumerator_block_call
+  private :__enumerator_block_call
 
   ##
   # call-seq:
@@ -574,7 +586,7 @@ class Enumerator
   #
   # Examples of usage:
   #
-  #   Enumerator.produce(1, &:succ)   # => enumerator of 1, 2, 3, 4, ....
+  #   Enumerator.produce(1, &:succ)   # => enumerator of 1, 2, 3, 4, ...
   #
   #   Enumerator.produce { rand(10) } # => infinite random number sequence
   #
